#include "function.h"

// ==============================
// = Funzioni per l'esercizio 2 =
// ==============================

// NOTA: la funzione "trovaMezzo" sara' utilissima anche per altri esercizi!

Mezzo* trovaMezzo(Mezzo* m, int dim, int codice) {
	// ciclo sull'array di mezzi fintanto che il codice non corrisponde (o
	// finche' non sono alla fine dell'array)
	int i = 0;
	for (; i < dim && m[i].codice != codice; ++i);
	// un po' di commenti:
	// for (; ... )   : l'istruzione di inizializzazione è assente
	// for (... i < dim && ... )   : prima condizione di terminazione
	// for (... && m[i].codice != codice ... )   : seconda condizione
	// for (... ; ++i)    : istruzione di incremento
	//
	// lo stesso risultato si poteva ottenere con:
	//
	// int i = 0;
	// while (i < dim && m[i].codice != codice)
	//     i++;
	//
	// che è più chiaro, ma un po' di C "tosto" prima o poi tocca che lo
	// vediate!
	
	
	// se sono arrivato alla fine dell'array, allora il codice era assente.
	// Altrimenti il mezzo con il codice richiesto è alla posizione "i"
	if (i < dim) return &m[i];
	else return NULL;
}

Prenotazione* trovaPrenotazione(Prenotazione* p, int dim, char* codice) {
	// identica alla precedente, ma funzione per le prenotazioni
	int i = 0;
	for (; i < dim && strcmp(p[i].codiceTrasporto, codice) != 0; ++i);
	
	if (i < dim) return &p[i];
	else return NULL;
}

int controlloTrasporto(Prenotazione* p, int p_dim,
					   Mezzo* m, int m_dim, char* cod)
{
	// NOTA: qui assumo che un codice di trasporto corrisponda sempre ad una
	// singola prenotazione
	
	// tento di recuperare la prenotazione con il codice specificato
	Prenotazione* pp = trovaPrenotazione(p, p_dim, cod);
	
	// se il codice non esiste, segnalo un errore
	if (pp == NULL) return -1;
	
	// inizio il calcolo del costo e della capacita' totali dei mezzi
	float costo_tot = 0;
	int cap_tot = 0;
	// ciclo sui mezzi prenotati
	int j;
	for (j = 0; j < pp->numeroMezzi; ++j) {
		// recupero il mezzo
		Mezzo* mm = trovaMezzo(m, m_dim, pp->mezzi[j]);
		// sommo la capacita' ed il costo
		costo_tot += mm->costo;
		cap_tot += mm->capacita;
	}
	
	// controllo che la capacita' ed il prezzo vadano bene
	if (pp->prezzo > costo_tot && pp->quantita <= cap_tot) return 1;
	return -1; // qui non ho messo "else", ma funziona lo stesso!
}

// ==============================
// = Funzioni per l'esercizio 3 =
// ==============================

Prenotazione creaPrenotazione(Mezzo* m, int m_dim)
{
	// praparo una struttura dati per i risultati
	Prenotazione res;
	
	printf("INSERIRE I DATI PER UNA NUOVA PRENOTAZIONE:\n");
	printf("codice trasporto: ");
	scanf("%s", res.codiceTrasporto);
	
	printf("anno/mese/giorno: ");
	scanf("%d/%d/%d", &res.anno, &res.mese, &res.giorno);
	
	printf("prezzo: ");
	scanf("%f", &res.prezzo);
	
	printf("quantita': ");
	scanf("%d", &res.quantita);
	
	do {
		printf("numero mezzi: ");
		scanf("%d", &res.numeroMezzi);
		if (res.numeroMezzi <= 0)
			printf("-- e' necessario richiedere almeno un mezzo\n");
	} while(res.numeroMezzi <= 0);
	
	// variabile per memorizzare il tipo dei mezzi nella prenotazione
	int tipo = -1;
	
	int i;
	for (i = 0; i < res.numeroMezzi; ++i) {
		// ci sono tre condizioni perche' un inserimento sia corretto:
		// cond1: il codice mezzo deve esistere
		// cond2: il codice mezzo non deve essere gia' stato inserito
		// cond3: il codice mezzo deve avere lo stesso tipo di quelli inseriti
		int cond1, cond2, cond3;
		do {		
			// richiedo un codice di mezzo
			int cod;
			printf("  codice mezzo #%d: ", i);
			scanf("%d", &cod);
			
			// Assumo che tutte la condizioni siano false
			cond1 = cond2 = cond3 = 0;
		
			// Verifico la condizione 1
			Mezzo* mm = trovaMezzo(m, m_dim, cod);
			cond1 = (mm != NULL); // metto in "cond1" il risultato del confronto
			
			// Se il mezzo esiste, verifico la condizione 2
			if (cond1) {
				// il codice non deve essere gia' stato inserito
				// NOTA: uso "cond2" come condizione addizionale nel ciclo. In
				// questo modo, appena la condizione diventa false il ciclo si
				// interrompe
				cond2 = 1;
				int j;
				for (j = 0; j < i && cond2; ++j)
					cond2 = (cod != res.mezzi[j]);
			}
			
			// Se fino a qua e' andato tutto bene, controllo la condizione 3
			if (cond1 && cond2) {
				// se non ho ancora inserito nessun mezzo, siamo a posto
				if (i == 0) cond3 = 1;
				else {
					// altrimenti, tutti i mezzi devono avere lo stesso tipo
					cond3 = 1;
					int j;
					for (j = 0; j < i && cond3; ++j)
						cond3 = (mm->tipo == tipo);
				}
			}
			
			// se tutto e' andato bene, posso inserire il mezzo
			if (cond1 && cond2 && cond3) {
				res.mezzi[i] = cod;
				tipo = mm->tipo; // memorizzo il tipo
			}
			
			if (!cond1)
				printf("  -- il codice specificato non esiste\n");
			if (cond1 && !cond2)
				printf("  -- codice gia' inserito\n");
			if (cond1 && cond2 && !cond3)
				printf("  -- non piu' di un tipo di mezzo\n");
			
		} while(!cond1 || !cond2 || !cond3);
	} // inserimento mezzi
	
	// restituisco la prenotazione
	return res;
}

// ==============================
// = Funzioni per l'esercizio 4 =
// ==============================

void salvaPrenotazione(Prenotazione p, char* nomeFile, Mezzo* m, int m_dim)
{
	// leggo l'elenco delle prenotazioni da file
	Prenotazione* pren;
	int npren;
	pren = readPrenotazione(nomeFile, &npren);
	
	// controllo che la prenotazione sia corretta mediante la funzione
	// "controlloTrasporto"
	int cond1;
	if (controlloTrasporto(&p, 1, m, m_dim, p.codiceTrasporto) == 1)
		cond1 = 1;
	else
		cond1 = 0;
	
	// assumo che la seconda condizione (niente mezzi condivisi) sia ok
	int cond2 = 1;
	// itero sull prenotazioni
	// NOTA: appena "cond2" diventa falsa posso interrompere il ciclo
	int i;
	for (i = 0; i < npren && cond2; ++i) {
		// controllo che il giorno coincida
		if (pren[i].anno == p.anno &&
			pren[i].mese == p.mese &&
			pren[i].giorno == p.giorno) {
			
			// la prenotazione e' corretta se nessuno dei mezzi richiesti
			// e' gia' utilizzato
			// NOTA: appena trovo un mezzo utilizzato in due prenotazioni
			// posso gia' concludere che "cond2" e' falsa
			int j, k;
			for (j = 0; j < p.numeroMezzi && cond2; ++j)
				for (k = 0; k < pren[i].numeroMezzi && cond2; ++k)
					cond2 = (p.mezzi[j] != pren[i].mezzi[k]);
		}
	} // fine iterazione sulle prenotazioni
	
	// se entrambe le condizioni sono state soddisfatte, posso salvare i dati
	if (cond1 && cond2) {
		FILE* fp;
		
		if ((fp = fopen(nomeFile, "a")) == NULL) {
			printf("Impossibile salvare i dati sul file \"%s\"\n", nomeFile);
			exit(1);
		}
		
		// scrivo i dati principali
		fprintf(fp, "%s %d %d %d %f %d %d", p.codiceTrasporto,
										    p.anno, p.mese, p.giorno,
										    p.prezzo, p.quantita,
											p.numeroMezzi);
		// scrivo i codici dei mezzi
		int i;
		for (i = 0; i < p.numeroMezzi; ++i)
			fprintf(fp, " %d", p.mezzi[i]);
		fprintf(fp, "\n");
		
		// chiudo il file
		fclose(fp);
	}
	
	if (!cond1)
		printf("Costo troppo alto o capacita' insufficiente\n");
	if (!cond2)
		printf("Alcuni mezzi sono gia' prenotati per questo giorno\n");
	
	// libero la memoria
	free(pren);
}