#include "gestione.h"

Mezzo* readMezzo(char* fileName, int* dim)
{	
	// Apertura del file di input
	FILE* fp = NULL;
	if ((fp = fopen(fileName, "r")) == NULL) {
		printf("Impossibile aprire il file \"%s\"\n", fileName);
		exit(1);
	}
	
	// lettura del numero di mezzi dal file
	fscanf(fp, "%d", dim);
	
	// allocazione dinamica di un array di mezzi
	Mezzo* mezzi = (Mezzo*) malloc((*dim) * sizeof(Mezzo));
	
	// lettura dei mezzi
	int i;
	for (i = 0; i < *dim; ++i) {
		int nr = fscanf(fp, "%d%d%d%f", &mezzi[i].codice, &mezzi[i].tipo,
							   			&mezzi[i].capacita, &mezzi[i].costo);
		// controllo che la lettura sia andata a buon fine
		if (nr < 4) {
			printf("Errore di lettura dal file \"%s\"\n", fileName);
			exit(1);
		}
	}
	
	// chiusura del file
	fclose(fp);
	
	// restituzione dell'array
	return mezzi;
}

void stampaMezzo(Mezzo m)
{
	printf("%d %d %d %f\n", m.codice, m.tipo, m.capacita, m.costo);
}


int leggiPrenotazione(FILE* fp, Prenotazione* p)
{
	// leggi i campi principali
	int nr = fscanf(fp, "%s%d%d%d%f%d%d", p->codiceTrasporto,
									      &p->anno, &p->mese, &p->giorno,
									      &p->prezzo,
									      &p->quantita,
									      &p->numeroMezzi);
	if (nr < 7) return 0;
	
	// leggi il numero di mezzi
	int i;
	for  (i = 0; i < p->numeroMezzi; ++i) {
		nr = fscanf(fp, "%d", &p->mezzi[i]);
		if (nr < 1) return 0;
	}
	
	// se siamo arrivati fino a qui, vuol dire che è andato tutto bene
	return 1;
}

Prenotazione* readPrenotazione(char* fileName, int* dim)
{
	// Apertura del file di input
	FILE* fp = NULL;
	if ((fp = fopen(fileName, "r")) == NULL) {
		printf("Impossibile aprire il file \"%s\"\n", fileName);
		exit(1);
	}
	
	// lettura delle prenotazioni (per stabilirne il numero)
	*dim = 0;
	Prenotazione temp;
	while (leggiPrenotazione(fp, &temp))
		++(*dim);

	// allocazione dinamica dell'array di prenotazioni
	Prenotazione* pren = (Prenotazione*) malloc((*dim) * sizeof(Prenotazione));
	
	// lettura dei dati
	rewind(fp);
	int i;
	for (i = 0; i < *dim; ++i)
		leggiPrenotazione(fp, &pren[i]);

	// chiusura del file
	fclose(fp);
	
	// restituzione dei dati
	return pren;
}

void stampaPrenotazione(Prenotazione p)
{
	// stampa i campi principali
	printf("%s %d %d %d %f %d %d", p.codiceTrasporto,
								   p.anno, p.mese, p.giorno,
								   p.prezzo, p.quantita, p.numeroMezzi);
	// stampa i codici dei mezzi prenotati
	int i;
	for (i = 0; i < p.numeroMezzi; ++i)
		printf(" %d", p.mezzi[i]);
	printf("\n");		   
}